const {
	postNotification,
	addNormalNotificationObserver,
	removeNotificationObserver
} = require("../utils/notificationCenter")
import {
	KEY_NOTIFICATION_LOGIN_SHOW,
	KEY_NOTIFICATION_LOGIN_SUCCESS
} from "../../static/keys/notification-keys.js"
import {
	readStorage,
	saveStorage
} from "../utils/storageUtil.js";
import User from "../models/User.js";
import {
	isEmpty,
	isNotEmpty,
} from "../utils/util.js";

import {
	PAKEAGE_NAME,
	VERSION_CODE
} from "../../utils/utils.js";
import {
	printError
} from "../utils/printUtil.js";
import config from "../../config/index.js";
import {
	toastMessage,
	toastHide
} from "../utils/toastUtil.js";

// 请求成功处理
function successHandler(url, param, res, callback) {
	let tokenExpireFn = () => {
		toastHide();

		if (uni.getStorageSync('showLoginModal')) {
			return
		}
		uni.setStorageSync('showLoginModal', true)
		uni.showModal({
			title: "登录状态已过期",
			content: "当前登录状态已过期，请重新登录！",
			cancelText: "暂不登录",
			confirmText: "前往登录",
			success: (res) => {
				if (res.confirm) {
					showLoginView();
				} else {
					logout(() => {}, true);
				}

				uni.setStorageSync('showLoginModal', false)
			}
		})
	}

	console.log(res)
	if (res.statusCode != 200) {
		toastMessage(`服务器异常,${res.statusCode}`)
		printError("请求失败", url, param, res.statusCode, res.errMsg);
		if (typeof callback == 'function') callback(false, res.errMsg)
	} else {
		if (res.data.status == 200) {
			if (typeof callback == 'function') callback(true, res.data.data)
		} else if (res.data.code == 401) {
			tokenExpireFn()
			if (typeof callback == 'function') callback(false, res.data.message);
		} else {
			toastMessage(`服务器异常,${res.data.code}`)
			printError("请求失败", url, param, res.data.code);
			if (typeof callback == 'function') callback(false, res.data.code)
		}
	}
}

// 请求失败处理
function failHandler(url, param, error, callback) {
	toastMessage("request fail~");
	printError("请求--失败", url, param, error.errMsg);
	if (typeof callback == 'function') callback(false, error.errMsg)
}

// 登录
function login(data, callback) {
	requestToken(data, (success, result) => {
		if (success) {

			saveToken(result.token); // 存储token
			refreshUserInfo(); // 更新用户数据


			setTimeout(() => {
				uni.reLaunch({
					url: `/pages/warehouse/warehouse?bookId=` + data.bookId,
				})

			}, 1400);
		}
	})
}

// 更新用户数据
function refreshUserInfo() {
	requestUserInfo((success, userData) => {
		if (success) {
			console.log(userData)
			let user = new User({
				...userData,
				name: userData.nickName,
				nickName: userData.nickName,
				avater: userData.avatar,
				id: userData.idcode,
				memberExpirationDate: userData.expireTime
			})
			saveNickname(userData.nickname);
			saveUserInfo(user); // 存储用户数据
			postNotification(KEY_NOTIFICATION_LOGIN_SUCCESS); // 通知登录成功
		}
	})
}

// 绑定手机号
function postPhone(data, callback) {
	let url = `${config["BASE_URL"]}/info/postPhone`;
	let header = {
		Authorization: `${readToken()}`
	}
	uni.request({
		url,
		header,
		data,
		method: "POST",
		success: (res) => {
			if (res.data.code == 200) {
				callback(true, "")
			} else {
				callback(false, "")
			}
		},
		fail: (error) => {
			callback(false, "")
		}
	})
}

// 请求token
function requestToken(data, callback) {
	let url = `${config["BASE_URL"]}/user/ttLogin`;
	let header = {
		pkgName: PAKEAGE_NAME,
		version: VERSION_CODE,
		token: ``
	}
	uni.request({
		url,
		data,
		header,
		method: "POST",
		success: (res) => {
			successHandler(url, data, res, callback)
		},
		fail: (error) => {
			failHandler(url, data, error, callback);
		}
	})
}

// 请求用户数据
function requestUserInfo(callback) {
	let url = `${config['BASE_URL']}/user/baseMsg`;
	let header = {}

	let uniChannel = 'wechat';
	// #ifdef MP-KUAISHOU
	uniChannel = 'kuaishou';
	// #endif

	// #ifdef MP-TOUTIAO
	uniChannel = 'douyin';
	// #endif

	Object.assign(header, {
		token: readToken(),
		pkgName: PAKEAGE_NAME,
		version: VERSION_CODE,
		proChannel: uniChannel
	})

	uni.request({
		url,
		header,
		method: "POST",
		success: (res) => {
			successHandler(url, {}, res, callback);
		},
		fail: (error) => {
			failHandler(url, data, error, callback);
		}
	})
}

// 退出登录
function logout(callback, exprireOut = false) {
	// let clearInfoFn = (msg) => {
	// 	if (msg) {
	// 		toastMessage(msg);
	// 	}
	// 	saveUserInfo(null);
	// 	saveToken(null)
	// }
	// if (exprireOut) {
	// 	clearInfoFn();
	// 	if (typeof callback == 'function') callback(true);
	// } else {
	// 	let url = `${config['BASE_URL']}/logout`;
	// 	let header = {
	// 		Authorization: `${readToken()}`
	// 	}
	// 	uni.request({
	// 		url,
	// 		header,
	// 		method: "GET",
	// 		success: (res) => {
	// 			successHandler(url, {}, res, (success, data) => {
	// 				if (success) {
	// 					clearInfoFn(data);
	// 				}
	// 				if (typeof callback == 'function') callback(success, data);
	// 			});
	// 		},
	// 		fail: (error) => {
	// 			failHandler(url, data, error, callback);
	// 		}
	// 	})
	// }
}

const KEY_STORAGE_TOKEN = "TOKEN"
let LOCAL_TOKEN;

function saveToken(token) {
	LOCAL_TOKEN = token;
	saveStorage(KEY_STORAGE_TOKEN, token);
}

function readToken() {
	if (!LOCAL_TOKEN) {
		// LOCAL_TOKEN = readStorage(KEY_STORAGE_TOKEN)
		LOCAL_TOKEN = readStorage('token')
	}
	return LOCAL_TOKEN;
}

const KEY_STORAGE_NICKNAME = "NICKNAME"
let LOCAL_NICKNAME;

function saveNickname(nickname) {
	LOCAL_NICKNAME = nickname;
	saveStorage(KEY_STORAGE_NICKNAME, nickname);
}

function readNickname() {
	if (!LOCAL_NICKNAME) {
		LOCAL_NICKNAME = readStorage(KEY_STORAGE_NICKNAME)
	}
	return LOCAL_NICKNAME;
}

const KEY_STORAGE_USERINFO = "USERINFO";
const KEY_NOTIFICATION_USERINFO_CHANGE = 'USERINFO_CHANGE';

function saveUserInfo(userInfo) {
	if (userInfo) {
		let result = readUserInfo() || {};
		Object.keys(userInfo).forEach(key => {
			result[key] = (userInfo[key] == null || userInfo[key] == undefined) ? result[key] : userInfo[key]
		})
		USERINFO = new User(result);
	} else {
		USERINFO = null;
	}
	saveStorage(KEY_STORAGE_USERINFO, USERINFO)
	postNotification(KEY_NOTIFICATION_USERINFO_CHANGE, {
		userInfo: USERINFO
	})

}
let USERINFO = null;

function readUserInfo() {
	if (isEmpty(USERINFO)) {
		USERINFO = readStorage(KEY_STORAGE_USERINFO);
	}
	USERINFO = isNotEmpty(USERINFO) ? new User(USERINFO) : null;
	return USERINFO;
}

function watchUserInfoChange(fn, observer) {
	if (typeof fn == 'function') fn.call(observer, {
		userInfo: readUserInfo()
	});
	addNormalNotificationObserver(KEY_NOTIFICATION_USERINFO_CHANGE, (info) => {
		if (typeof fn == 'function') fn.call(observer, {
			userInfo: info.userInfo
		})
	}, observer)
}

function removeUserInfoChangeWatch(observer) {
	removeNotificationObserver(KEY_NOTIFICATION_USERINFO_CHANGE, observer);
}

// 展示登录弹窗
function showLoginView() {
	postNotification(KEY_NOTIFICATION_LOGIN_SHOW, {
		show: true
	})
}

module.exports = {
	login,
	refreshUserInfo,
	postPhone,
	requestToken,
	requestUserInfo,
	logout,
	showLoginView,
	saveUserInfo,
	readUserInfo,
	readNickname,
	watchUserInfoChange,
	removeUserInfoChangeWatch,
	saveToken,
	readToken,
}