import {
	navigateTo,
	redirectTo,
	loading,
	message,
	confirm,
	alert
} from '../utils/fun.js';

import {
	apiPOST
} from '../common/utils/apiRequest.js'

const app = getApp();

export default {
	data() {
		return {
			xhrPool: new Set(),
			bottomSafePadding: app.globalData.bottomSafePadding,
			options: {},
			auth: false, // 登录验证
			paying: false, // 支付按钮状态
		};
	},
	onLoad(options) {
		this.options = options;

	},
	onShow() {
		this.loadData();
	},
	onHide() {
		if (this.xhrPool.size) {
			this.xhrPool.forEach((requestTask) => {
				requestTask.abort();
			})
		}
	},
	methods: {
		authTo(url) { //登录校验

		},
		authToNs(url) { //登录校验不保留当前页面

		},
		loadData() {},
		startPay() {
			loading.show();
			this.paying = true;
		},
		stopPay() {
			loading.hide();
			this.paying = false;
		},
		/**
		 * @param {Object} url 接口请求地址
		 * @param {Object} method 接口请求方式
		 * @param {Object} data 接口请求参数
		 */
		$http(url, method, data = {}, showLoading = false) {
			return new Promise((resolve, reject) => {
				this[method.toLowerCase()]({
					url: url,
					data: data,
					showLoading: showLoading,
					success: (res) => {
						resolve(res)
					},
					fail: (e) => {
						reject(e)
					}
				});
			})
		},
		login() {
			// navigateTo('user/login');
			// let redirect = this.$scope.$page.fullPath.replace('/pages', '');
			// console.log(redirect)
			// navigateTo('user/login?redirect=' + encodeURIComponent(redirect));
		},
		logout() {
			uni.removeStorageSync('token');
		},
		post(options) {
			options = Object.assign({
				showLoading: true
			}, options, {
				method: 'POST'
			})
			this.req(options);
		},
		// get(options) {
		// 	options = Object.assign({
		// 		showLoading: false
		// 	}, options, {
		// 		method: 'GET'
		// 	})
		// 	this.req(options);
		// },
		req(options) {
			let {
				showLoading,
				url,
				data,
				auth
			} = options;

			const success = options?.success;
			const fail = options?.fail;
			const header = options?.header ?? {};

			showLoading && loading.show();

			apiPOST({
				url: `${url}`,
				data: data,
				callback: (_success, _data) => {
					showLoading && loading.hide();

					if (_success) {
						success({
							data: _data
						})
					} else {
						fail(_data)
						message.notify(res.data.message);
					}
				}
			})
		}
	}
};