/**
 * 交互反馈
 */
function toastFun(icon) {
	return function(title) {
		uni.showToast({
			title,
			icon
		});
	}
}

//获取随机id
export function uuid(len, binary) {
	len = !len ? 36 : len;
	binary = !binary ? 16 : binary;
	return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
		var r = Math.random() * binary | 0,
			v = c == 'x' ? r : (r & 0x3 | 0x8);
		return v.toString(binary);
	}).substring(0, len)
};

export const message = {
	error: toastFun('error'),
	success: toastFun('success'),
	fail: toastFun('fail'),
	exception: toastFun('exception'),
	notify: toastFun('none'),
	hide: uni.hideToast
};

export let loading = {
	show(title = '') {
		uni.showLoading({
			title,
			mask: true
		});
	},

	hide: uni.hideLoading
};

export function alert(options) {
	options = Object.assign({
		title: '',
		content: '',
		confirmText: '确定',
		confirmColor: '#03E7C9',
		fail(e) {
			console.log(e);
		}
	}, options, {
		showCancel: false
	});

	uni.showModal(options);
}
export function confirm(options) {
	const success = options?.success;
	const cancel = options?.cancel;

	options = Object.assign({
		title: '',
		confirmText: '确定',
		cancelText: '取消',
		confirmColor: '#03E7C9',
	}, options, {
		success(res) {
			if (res.confirm) {
				if (success) {
					success();
				}
			} else {
				if (cancel) {
					cancel();
				}
			}
		},
		fail() {
			console.log(e);
			if (cancel) {
				cancel();
			}
		}
	});

	uni.showModal(options);
}

/**
 * 数据缓存
 */
// export function getStorage(key, success, fail, complete) {
// 	uni.getStorage({
// 		key,
// 		success(res) {
// 			if (success) {
// 				success.call(this, res.data);
// 			}
// 		},
// 		fail,
// 		complete
// 	});
// }
// export function removeStorage(key, success, fail, complete) {
// 	uni.removeStorage({
// 		key,
// 		success(res) {
// 			if (success) {
// 				success.call(this, res.data);
// 			}
// 		},
// 		fail,
// 		complete
// 	});
// }
// export function setStorage(key, data, success, fail, complete) {
// 	uni.setStorage({
// 		key,
// 		data,
// 		success,
// 		fail,
// 		complete
// 	});
// }

/**
 * 路由与页面跳转
 */
function routeFun(url, params = {}, success, fail, complete) {
	if (url.includes(`/pages`) == -1) {
		url = `/pages/${url}`;
	}
	let query = [];

	for (var p in params) {
		if (params.hasOwnProperty(p)) {
			query.push(encodeURIComponent(p) + '=' + encodeURIComponent(params[p]));
		}
	}
	if (query.length) {
		url = `${url}?${query.join('&')}`;
	}

	this({
		url,
		success,
		fail,
		complete
	});
}

export const switchTab = routeFun.bind(uni.switchTab);
export const reLaunch = routeFun.bind(uni.reLaunch);
export const redirectTo = routeFun.bind(uni.redirectTo);
export const navigateTo = routeFun.bind(uni.navigateTo);
export function navigateBack(delta = 1) {
	uni.navigateBack({
		delta
	});
}

// export function request(url, data, method, header, success, fail, complete) {
// 	return uni.request({
// 		url,
// 		data,
// 		header,
// 		method,
// 		success,
// 		fail,
// 		complete
// 	});
// }

function commonFun(success, fail, complete) {
	this({
		provider: 'weixin',
		success,
		fail,
		complete
	});
}
export const login = commonFun.bind(uni.login);

// export const chooseImage = routeFun.bind(wx.chooseImage)
export function chooseImage(success, fail, complete) {
	uni.chooseImage({
		count: 1,
		sizeType: 'compressed',
		success,
		fail,
		complete
	});
}

export function compressImage(src, success, fail, complete) {
	uni.compressImage({
		src,
		quality: 50,
		success,
		fail,
		complete
	})
}
// export function getFileSystemManager() {
//   return wx.getFileSystemManager()
// }
// export function arrayBufferToBase64(arrayBuffer) {
//   return wx.arrayBufferToBase64(arrayBuffer)
// }

export function downloadFile(url, success, fail, complete) {
	uni.downloadFile({
		url,
		success,
		fail,
		complete
	})
}

export function openDocument(filePath, success, fail, complete) {
	uni.openDocument({
		filePath,
		showMenu: true,
		fileType: "pdf",
		success,
		fail,
		complete
	})
}