
export const PAKEAGE_NAME = 'com.duben.dybookhm'
export const VERSION_CODE = '1.0.1'
// export const PAKEAGE_NAME = 'com.test.test'
 
export function trim(str) {
	return str ? str.replace(/(^\s*)|(\s*$)/g, "") : ''
}

// 处理多次点击
export function noMultipleClicks(methods) {
	let that = this;

	if (that.noClick) {
		that.noClick = false;
		methods();
		setTimeout(function() {
			that.noClick = true;
		}, 2000)
	} else {
		// TODO toast
	}
}

// 防抖函数
export function debounce(func, wait = 800, immediate = false) {
	let timer = null,
		result;
	let debounced = function(...args) {
		if (timer) {
			clearTimeout(timer);
		}

		if (immediate) {
			let callNow = !timer;
			timer = setTimeout(() => {
				timer = null;
			}, wait);
			if (callNow) {
				result = func.apply(this, args);
			}
		} else {
			timer = setTimeout(() => {
				func.apply(this, args);
			}, wait);
		}
		return result;
	};

	debounced.cancel = () => {
		clearTimeout(timer);
		timer = null;
	};

	return debounced;
}

// 节流函数
export function throttle(func, wait = 800, ...args) {
	let timer;
	return function() {
		if (!timer) {
			timer = setTimeout(() => {
				timer = null;
				func.apply(this, args);
			}, wait)
		}
	}
}

/**
 * 加法
 * @param arg1
 * @param arg2
 * @returns
 */
export function accAdd(arg1, arg2) {
	let r1, r2, m;
	try {
		r1 = arg1.toString().split(".")[1].length
	} catch (e) {
		r1 = 0
	}

	try {
		r2 = arg2.toString().split(".")[1].length
	} catch (e) {
		r2 = 0
	}

	m = Math.pow(10, Math.max(r1, r2));
	return (arg1 * m + arg2 * m) / m;
}

/**
 * 减法
 * @param arg1
 * @param arg2
 * @returns
 */
export function accSubtr(arg1, arg2) {
	let r1, r2, m, n;
	try {
		r1 = arg1.toString().split(".")[1].length;
	} catch (e) {
		r1 = 0;
	}
	try {
		r2 = arg2.toString().split(".")[1].length;
	} catch (e) {
		r2 = 0;
	}
	m = Math.pow(10, Math.max(r1, r2));
	//动态控制精度长度
	n = (r1 >= r2) ? r1 : r2;
	return ((arg1 * m - arg2 * m) / m).toFixed(n);
}

/***
 * 乘法，获取精确乘法的结果值
 * @param arg1
 * @param arg2
 * @returns
 */
export function accMul(arg1, arg2) {
	let m = 0,
		s1 = arg1.toString(),
		s2 = arg2.toString();
	try {
		m += s1.split(".")[1].length
	} catch (e) {}
	try {
		m += s2.split(".")[1].length
	} catch (e) {}
	return Number(s1.replace(".", "")) * Number(s2.replace(".", "")) / Math.pow(10, m);
}

/***
 * 除法，获取精确乘法的结果值
 * @param arg1
 * @param arg2
 * @returns
 */
export function accDivCoupon(arg1, arg2) {
	let t1 = 0,
		t2 = 0,
		r1, r2;
	try {
		t1 = arg1.toString().split(".")[1].length;
	} catch (e) {}
	try {
		t2 = arg2.toString().split(".")[1].length;
	} catch (e) {}
	r1 = Number(arg1.toString().replace(".", ""));
	r2 = Number(arg2.toString().replace(".", ""));
	return (r1 / r2) * Math.pow(10, t2 - t1);
}

export function getQuery(keys) {
	let query = window.location.search.substring(1);
	let vars = query.split("&");
	for (let i = 0; i < vars.length; i++) {
		let pair = vars[i].split("=");
		if (pair[0] === keys) {
			return pair[1];
		}
	}
	return '';
}

export function getQueryByUrl(url, keys) {
	let i = url.indexOf('?') + 1
	let vars = url.substring(i, url.length).split('&');
	for (let i = 0; i < vars.length; i++) {
		let pair = vars[i].split("=");
		if (pair[0] === keys) {
			return pair[1];
		}
	}
	return '';
}

export function imgCompress(file) {
	return new Promise((resolve, reject) => {
		if (!window.FileReader) {
			reject('浏览器对FileReader方法不兼容');
		} else {
			let reader = new FileReader();
			reader.readAsDataURL(file); //读出 base64
			reader.onloadend = () => {
				let img = new Image();
				img.src = reader.result;
				img.onload = () => {
					let w = img.naturalWidth,
						h = img.naturalHeight,
						resizeW = 0,
						resizeH = 0;
					let maxSize = {
						width: 1000,
						height: 1000,
						level: 0.2
					};
					if (w > maxSize.width || h > maxSize.height) {
						let multiple = Math.max(w / maxSize.width, h / maxSize.height);
						resizeW = w / multiple;
						resizeH = h / multiple;
						let canvas = document.createElement('canvas'),
							ctx = canvas.getContext('2d');
						canvas.width = resizeW;
						canvas.height = resizeH;
						ctx.drawImage(img, 0, 0, resizeW, resizeH);
						let base64 = canvas.toDataURL('image/jpeg', maxSize.level);
						resolve(base64);
					} else {
						// 如果图片尺寸小于最大限制，则不压缩直接上传
						resolve(reader.result);
					}
				}
			};
		}
	});
}

export function px2rpx(px) {
	return (px / (uni.upx2px(px) / px))
}

export function dataURLtoFile(dataurl, filename) {
	var arr = dataurl.split(','),
		mime = arr[0].match(/:(.*?);/)[1],
		bstr = atob(arr[1]),
		n = bstr.length,
		u8arr = new Uint8Array(n);
	while (n--) {
		u8arr[n] = bstr.charCodeAt(n);
	}
	return new File([u8arr], filename, {
		type: mime
	});
}

// 图片压缩递归，小于1M跳出
export function imageCompress(file) {
	return new Promise((resolve, reject) => {
		let {
			size,
			path
		} = file
		let type = path.split(".")[1]
		//大于1M进行压缩，
		if (size < (1024 * 1024)) {
			resolve(file)
			return false
		}
		uni.compressImage({
			src: path,
			quality: 80,
			success: res => {
				let newPath = res.tempFilePath + type
				let newName = res.tempFilePath.split("/")[res.tempFilePath.split("/").length - 1] +
					type
				uni.getFileInfo({
					filePath: res.tempFilePath,
					success: async (info) => {
						let newFile = {
							...file,
							size: info.size,
							path: newPath,
							name: newName,
							tempFilePath: res.tempFilePath
						}
						resolve(await imageCompress(newFile))
					}
				})
			}
		})

	})
}

//压缩base64图片
export function base64Compress(base64String, w, quality) {

	var getMimeType = (urlData) => {
		var arr = urlData.split(",");
		var mime = arr[0].match(/:(.*?);/)[1];
		// return mime.replace("image/", "");
		return mime;
	};
	var newImage = new Image();
	var imgWidth, imgHeight;

	var promise = new Promise((resolve) => (newImage.onload = resolve));
	newImage.src = base64String;
	return promise.then(() => {
		imgWidth = newImage.width;
		imgHeight = newImage.height;
		var canvas = document.createElement("canvas");
		var ctx = canvas.getContext("2d");
		if (Math.max(imgWidth, imgHeight) > w) {
			if (imgWidth > imgHeight) {
				canvas.width = w;
				canvas.height = (w * imgHeight) / imgWidth;
			} else {
				canvas.height = w;
				canvas.width = (w * imgWidth) / imgHeight;
			}
		} else {
			canvas.width = imgWidth;
			canvas.height = imgHeight;
		}
		ctx.clearRect(0, 0, canvas.width, canvas.height);
		ctx.drawImage(newImage, 0, 0, canvas.width, canvas.height);
		var base64 = canvas.toDataURL(getMimeType(base64String), quality);
		console.log(base64);
		return base64;
	});
}

export function getSubList(length, list) {
	var size = list.length
	var temp = size / length + 1
	var result = (size % length) == 0
	var subList = []
	for (let i = 0; i < temp; i++) {
		if (i == temp - 1) {
			if (result) {
				break
			}
			let tempList = list.slice(length * i, size)
			if (tempList.length > 0) {
				subList.push(tempList)
			}
		} else {
			let tempList = list.slice(length * i, length * (i + 1))
			if (tempList.length > 0) {
				subList.push(tempList)
			}
		}
	}
	return subList
}

export function formateTime(dateTime, fmt) {
	let date = new Date(dateTime)
	if (/(y+)/.test(fmt)) {
		fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
	}
	let o = {
		'M+': (date.getMonth() + 1).toString().padStart(2, '0'),
		'd+': date.getDate().toString().padStart(2, '0'),
		'h+': date.getHours(),
		'm+': date.getMinutes(),
		's+': date.getSeconds()
	};
	for (let k in o) {
		if (new RegExp(`(${k})`).test(fmt)) {
			let str = o[k] + '';
			fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? str : str);
		}
	}
	return fmt;
}